/*
 * gsampler-client-database.c
 * Copyright (C) Lilian Monchalin 2011 <isaya07@users.sf.net>
 * 
 * gsampler is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * gsampler is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include <string.h>

#include "gsampler-client-database.h"
#include "gsampler-utils.h"

//DB INST INFO
void 
gsampler_db_inst_info_copy (const GsamplerDbInstInfo* self, GsamplerDbInstInfo* dest) 
{
	dest->name = g_strdup (self->name);
	dest->db_path = g_strdup (self->db_path);
	dest->file = g_strdup (self->file);
	dest->number = self->number;
	dest->format_family = g_strdup (self->format_family);
	dest->format_version = g_strdup (self->format_version);
	dest->size = self->size;
	dest->type = g_strdup (self->type);
	dest->created = g_strdup (self->created);
	dest->modified = g_strdup (self->modified);
	dest->desc = g_strdup (self->desc);
	dest->product = g_strdup (self->product);
	dest->artists = g_strdup (self->artists);
	dest->keywords = g_strdup (self->keywords);
}

GsamplerDbInstInfo* 
gsampler_db_inst_info_dup (const GsamplerDbInstInfo* self) 
{
	GsamplerDbInstInfo* dup;
	dup = g_new0 (GsamplerDbInstInfo, 1);
	gsampler_db_inst_info_copy (self, dup);
	return dup;
}

void 
gsampler_db_inst_info_destroy (GsamplerDbInstInfo* self) 
{
	g_free (self->file);
	g_free (self->format_family);
	g_free (self->format_version);
	g_free (self->type);
	g_free (self->created);
	g_free (self->modified);
	g_free (self->desc);
	g_free (self->product);
	g_free (self->artists);
	g_free (self->keywords);
}

void 
gsampler_db_inst_info_free (GsamplerDbInstInfo* self) 
{
	gsampler_db_inst_info_destroy (self);
	g_free (self);
}

GType
gsampler_db_inst_info_get_type (void)
{
	static volatile gsize gsampler_db_inst_info_type_id__volatile = 0;
	if (g_once_init_enter (&gsampler_db_inst_info_type_id__volatile)) {
		GType gsampler_db_inst_info_type_id;
		gsampler_db_inst_info_type_id = g_boxed_type_register_static ("GsamplerDbInstInfo", (GBoxedCopyFunc) gsampler_db_inst_info_dup, (GBoxedFreeFunc) gsampler_db_inst_info_free);
		g_once_init_leave (&gsampler_db_inst_info_type_id__volatile, gsampler_db_inst_info_type_id);
	}
	return gsampler_db_inst_info_type_id__volatile;
}

//DB JOB INFO
void 
gsampler_db_job_info_copy (const GsamplerDbJobInfo* self, GsamplerDbJobInfo* dest) 
{
	dest->total = self->total;
	dest->scanned = self->scanned;
	dest->scanning = g_strdup (self->scanning);
	dest->status = self->status;
}

GsamplerDbJobInfo* 
gsampler_db_job_info_dup (const GsamplerDbJobInfo* self) 
{
	GsamplerDbJobInfo* dup;
	dup = g_new0 (GsamplerDbJobInfo, 1);
	gsampler_db_job_info_copy (self, dup);
	return dup;
}

void 
gsampler_db_job_info_destroy (GsamplerDbJobInfo* self) 
{
	g_free (self->scanning);
}

void 
gsampler_db_job_info_free (GsamplerDbJobInfo* self) 
{
	gsampler_db_job_info_destroy (self);
	g_free (self);
}

/**
 *	DATA BASE STUFF
 * 
 **/

gchar** 
gsampler_client_db_list_dir (GsamplerClient* self, const gchar* path) {
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (path != NULL, NULL);

	return gsampler_client_get_list (self, g_strconcat ("LIST DB_INSTRUMENT_DIRECTORIES ", "'", path, "'", NULL));
}

gchar** 
gsampler_client_db_list_instrument (GsamplerClient* self, const gchar* path) {

	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (path != NULL, NULL);

	return gsampler_client_get_list (self, g_strconcat ("LIST DB_INSTRUMENTS ", "'", path, "'", NULL));
}

GsamplerDbInstInfo * 
gsampler_client_db_get_instrument_info (GsamplerClient* self, const gchar* path) 
{
	GsamplerDbInstInfo* result = NULL;
	gchar* com;
	GSList* ret;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (path != NULL, NULL);

	result = g_new0 (GsamplerDbInstInfo, 1);
	com = g_strconcat ("GET DB_INSTRUMENT INFO ", "'", path, "'", NULL);
	ret = gsampler_client_client_get_info (self, com);
	if (g_slist_length (ret) != 0) {
		GSList* ele_it;
		for (ele_it = ret; ele_it != NULL; ele_it = ele_it->next) {
			gchar* ele;
			ele = g_strdup ((const gchar*) ele_it->data);
			if (string_contains (ele, "INSTRUMENT_FILE")) {
				result->file = g_strstrip (string_replace (ele, "INSTRUMENT_FILE:", ""));
			} else if (string_contains (ele, "INSTRUMENT_NR")) {
				gchar* temp = NULL;
				temp = g_strstrip (string_replace (ele, "INSTRUMENT_NR:", ""));
				result->number = atoi (temp);
				g_free (temp);
			} else if (string_contains (ele, "FORMAT_FAMILY")) {
				result->format_family = g_strstrip (string_replace (ele, "FORMAT_FAMILY:", ""));
			} else if (string_contains (ele, "FORMAT_VERSION")) {
				result->format_version = g_strstrip (string_replace (ele, "FORMAT_VERSION:", ""));
			} else if (string_contains (ele, "SIZE")) {
				gchar* temp = NULL;
				temp = g_strstrip (string_replace (ele, "SIZE:", ""));
				int value = atoi (temp);
				result->size = (float) value / (1024 * 1024);
				g_free (temp);
			} else if (string_contains (ele, "IS_DRUM")) {
				gboolean temp = bool_parse (g_strstrip (string_replace (ele, "IS_DRUM:", "")));
				if (!temp)
					result->type = g_strdup ("Chromatic Instrument");
				else
					result->type = g_strdup ("Drumkit"); 
			} else if (string_contains (ele, "CREATED")) {
				result->created = g_strstrip (string_replace (ele, "CREATED:", ""));
			} else if (string_contains (ele, "MODIFIED")) {
				result->modified = g_strstrip (string_replace (ele, "MODIFIED:", ""));
			} else if (string_contains (ele, "DESCRIPTION")) {
				result->desc = g_strstrip (string_replace (ele, "DESCRIPTION:", ""));
			} else if (string_contains (ele, "PRODUCT")) {
				result->product = g_strstrip (string_replace (ele, "PRODUCT:", ""));
			} else if (string_contains (ele, "ARTISTS")) {
				result->artists = g_strstrip (string_replace (ele, "ARTISTS:", ""));
			} else if (string_contains (ele, "KEYWORDS")) {
				result->keywords = g_strstrip (string_replace (ele, "KEYWORDS:", ""));
			} 
			g_free (ele);
		}
	} else {
		result->file = g_strdup ("NONE");
	}
	g_slist_free_full (ret, g_free);
	g_free (com);
	return result;
}

gboolean 
gsampler_client_db_format_instruments (GsamplerClient* self) 
{
	gchar* ret = NULL;
	g_return_val_if_fail (self != NULL, FALSE);
	
	ret = gsampler_client_send_commande (self, "FORMAT INSTRUMENTS_DB");
	if (!string_contains (ret, "OK")) {
		g_free (ret);
		return FALSE;
	}
	g_free (ret);
	return TRUE;
}

gint 
gsampler_client_db_add_instrument (GsamplerClient* self, const gchar* db_base, const gchar* path, 
                                   gboolean recur, gboolean flat, gboolean as_dir, gint index) 
{
	gint result;
	gchar* com;
	g_return_val_if_fail (self != NULL, 0);
	g_return_val_if_fail (db_base != NULL, 0);
	g_return_val_if_fail (path != NULL, 0);

	com = g_strdup ("ADD DB_INSTRUMENTS NON_MODAL ");
	if (recur) {
		com = g_strconcat (com, "RECURSIVE ", NULL);
	}
	if (flat) {
		com = g_strconcat (com, "FLAT ", NULL);
	}
	if (as_dir) {
		com = g_strconcat (com, "FILE_AS_DIR ", NULL);
	}
	com = g_strconcat (com, "'", db_base,"' '", path, "'", NULL);
	if (index != -1) {
		com = g_strconcat (com, " ", g_strdup_printf ("%i", index), NULL);
	}
	result = gsampler_client_create (self, com);
	g_free (com);
	return result;
}

gchar**
gsampler_client_db_find_instrument(GsamplerClient* self, const gchar* dir, const gchar* name)
{
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (dir != NULL, NULL);
	g_return_val_if_fail (name != NULL, NULL);
	
	return gsampler_client_get_list (self, g_strdup_printf ("FIND DB_INSTRUMENTS '%s' NAME='%s'", dir, name));
}

GsamplerDbJobInfo * 
gsampler_client_db_get_job_info (GsamplerClient* self, gint job) 
{
	GsamplerDbJobInfo* result = NULL;
	GSList* ret;
	g_return_val_if_fail (self != NULL, NULL);

	result = g_new0 (GsamplerDbJobInfo, 1);
	ret = gsampler_client_client_get_info (self, g_strdup_printf ("GET DB_INSTRUMENTS_JOB INFO %i", job));
	if (g_slist_length (ret) != 0) {
		GSList* ele_it;
		for (ele_it = ret; ele_it != NULL; ele_it = ele_it->next) {
			gchar* ele;
			ele = g_strdup ((const gchar*) ele_it->data);
			if (string_contains (ele, "FILES_TOTAL")) {
				gchar* temp;
				temp = string_replace (ele, "FILES_TOTAL: ", "");;
				result->total = double_parse (temp);
				g_free (temp);
			} else if (string_contains (ele, "FILES_SCANNED")) {
				gchar* temp;
				temp = string_replace (ele, "FILES_SCANNED: ", "");
				result->scanned = double_parse (temp);
				g_free (temp);
			} else if (string_contains (ele, "SCANNING")) {
				result->scanning = string_replace (ele, "SCANNING: ", "");
			} else if (string_contains (ele, "STATUS")) {
				gchar* temp;
				temp = string_replace (ele, "STATUS: ", "");
				result->status = double_parse (temp);
				g_free (temp);
			}
			g_free (ele);
		}
	} else {
		result->status = (gdouble) (-1);
	}
	g_slist_free_full (ret, g_free);
	return result;
}

gboolean
gsampler_client_db_move_dir (GsamplerClient* self, const gchar* dir, const gchar* dst)
{
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (dir != NULL, FALSE);
	g_return_val_if_fail (dst != NULL, FALSE);
	
	return gsampler_client_send_bool (self, g_strdup_printf("MOVE DB_INSTRUMENT_DIRECTORY '%s' '%s'", dir, dst));
}

gboolean
gsampler_client_db_copy_dir (GsamplerClient* self, const gchar* dir, const gchar* dst)
{
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (dir != NULL, FALSE);
	g_return_val_if_fail (dst != NULL, FALSE);
	
	return gsampler_client_send_bool (self, g_strdup_printf("COPY DB_INSTRUMENT_DIRECTORY '%s' '%s'", dir, dst));

}

gboolean
gsampler_client_db_remove_dir (GsamplerClient* self, const gchar* dir)
{
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (dir != NULL, FALSE);
	
	return gsampler_client_send_bool (self, g_strdup_printf("REMOVE DB_INSTRUMENT_DIRECTORY FORCE '%s'", dir));
}

gboolean
gsampler_client_db_rename_dir (GsamplerClient* self, const gchar* dir, const gchar* name)
{
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (dir != NULL, FALSE);
	g_return_val_if_fail (name != NULL, FALSE);	
	
	return gsampler_client_send_bool (self, g_strdup_printf("SET DB_INSTRUMENT_DIRECTORY NAME '%s' '%s'", dir, name));

}

gchar**
gsampler_client_db_check_lost_inst_files (GsamplerClient* self)
{
	g_return_val_if_fail (self != NULL, NULL);

	return gsampler_client_get_list (self, "FIND LOST DB_INSTRUMENT_FILES");
}

gboolean
gsampler_client_db_copy_instrument (GsamplerClient* self,
                                    const gchar* instr, const gchar* dst)
{
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (instr != NULL, FALSE);
	g_return_val_if_fail (dst != NULL, FALSE);
	
	return gsampler_client_send_bool (self, g_strdup_printf("COPY DB_INSTRUMENT '%s' '%s'", instr, dst));
}

gboolean
gsampler_client_db_move_instrument (GsamplerClient* self, 
                                    const gchar* instr, const gchar* dst)
{
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (instr != NULL, FALSE);
	g_return_val_if_fail (dst != NULL, FALSE);
	
	return gsampler_client_send_bool (self, g_strdup_printf("MOVE DB_INSTRUMENT '%s' '%s'", instr, dst));
}

gboolean
gsampler_client_db_remove_instrument (GsamplerClient* self, const gchar* instr)
{
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (instr != NULL, FALSE);
	
	return gsampler_client_send_bool (self, g_strdup_printf("REMOVE DB_INSTRUMENT '%s'", instr));
}

gboolean
gsampler_client_db_rename_instrument (GsamplerClient* self, const gchar* instr, const gchar* name)
{
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (instr != NULL, FALSE);
	g_return_val_if_fail (name != NULL, FALSE);	
	
	return gsampler_client_send_bool (self, g_strdup_printf("SET DB_INSTRUMENT NAME '%s' '%s'", instr, name));
}

gboolean
gsampler_client_db_set_instrument_desc (GsamplerClient *self, 
                                        const gchar* instr, const gchar* desc)
{
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (instr != NULL, FALSE);
	g_return_val_if_fail (desc != NULL, FALSE);
	
	return gsampler_client_send_bool (self, g_strdup_printf("SET DB_INSTRUMENT DESCRIPTION '%s' '%s'", instr, desc));
}
