/*
 * gsampler-window.h
 * Copyright (C) Lilian Monchalin 2011 <isaya07@users.sf.net>
 * 
 * gsampler is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * gsampler is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _GSAMPLER_WINDOW_H_
#define _GSAMPLER_WINDOW_H_

#include <gtk/gtk.h>
#include "gsampler-client.h"
#include "gsampler-uimanager.h"
#include "gsampler-channel-manager.h"
#include "gsampler-device-manager.h"
#include "gsampler-midi-map.h"
#include "gsampler-statusbar.h"
#include "gsampler-option.h"
#include "gsampler-console.h"
#include "gsampler-keyboard.h"
#include "gsampler-session.h"
#include "gsampler-database.h"
G_BEGIN_DECLS

#define GSAMPLER_TYPE_MAIN_WINDOW (gsampler_main_window_get_type ())
#define GSAMPLER_MAIN_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GSAMPLER_TYPE_MAIN_WINDOW, GsamplerMainWindow))
#define GSAMPLER_MAIN_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GSAMPLER_TYPE_MAIN_WINDOW, GsamplerMainWindowClass))
#define GSAMPLER_IS_MAIN_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GSAMPLER_TYPE_MAIN_WINDOW))
#define GSAMPLER_IS_MAIN_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GSAMPLER_TYPE_MAIN_WINDOW))
#define GSAMPLER_MAIN_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GSAMPLER_TYPE_MAIN_WINDOW, GsamplerMainWindowClass))
#define GSAMPLER_MAIN_WINDOW_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GSAMPLER_TYPE_MAIN_WINDOW, GsamplerMainWindowPrivate))

typedef struct _GsamplerMainWindow GsamplerMainWindow;
typedef struct _GsamplerMainWindowClass GsamplerMainWindowClass;
typedef struct _GsamplerMainWindowPrivate GsamplerMainWindowPrivate;

struct _GsamplerMainWindow {
	GtkWindow parent_instance;
	GsamplerMainWindowPrivate * priv;
};

struct _GsamplerMainWindowClass {
	GtkWindowClass parent_class;
};

struct _GsamplerMainWindowPrivate {
	GtkAdjustment* main_vol;
	GtkDrawingArea* master_draw;
	GtkPaned* paned1;
	GtkPaned* paned2;
	GtkNotebook* notebook;
	gchar* font;
	gchar* total_stream;
	gchar* total_voice;
	GtkMenuBar* _menu;
	GtkToolbar* _toolbar;
 	GsamplerStatusBar* _statusbar;
	GsamplerKeyboard* _keyboard;
	GsamplerUiManager* _ui_manag;
	GsamplerConsole* console;
	GsamplerSession* _session;
	GsamplerOption* _option;
	GsamplerDeviceManager* _dev_manag;
	GsamplerChannelManager* _chan_manag;
	GsamplerMidiMap* _midi_map;
	GsamplerDataBase* _data_base;
	gint draw_idle_id;
	gboolean status;
};

GType 				gsampler_main_window_get_type (void) G_GNUC_CONST;
GsamplerMainWindow* gsampler_main_window_new (void);
GsamplerMainWindow* gsampler_main_window_get_instance (void);

void 				gsampler_main_window_set_status_progress (GsamplerMainWindow* self, 
				                               				  const gchar* type, gdouble frac, 
				                               				  const gchar* mes, gpointer data);
void				gsampler_main_window_set_status_pulse (GsamplerMainWindow* self, gboolean watch);
gboolean 			gsampler_main_window_start_client (gpointer data);
void 				gsampler_main_window_send_message (const gchar* level, const gchar* message);
void 				gsampler_main_window_set_wait_cursor (gboolean state);
void 				gsampler_main_window_set_hand_cursor (gboolean state);
gdouble 			gsampler_main_window_get_master_volume (void);
GdkWindow* 			gsampler_main_window_get_gdk_window (void);
gboolean			gsampler_man_window_status (void);
G_END_DECLS

#endif /* _GSAMPLER_WINDOW_H_ */
